clear; clc; close all;

%addpath(genpath('d:\seg')); M = [];

f = double(imread('3Tbrain2.png')); load M; 
f = f(:,:,1);f = f./max(f(:)); 
nPh = 4;
wLength = 1e-4;
relchg_tol = 1e-3;
maxiter = 100;
tau = .5;
theta = .2;

%% Initialization
[m, n] = size(f); b = ones(m,n); %if isempty(M), M = ones(m,n); end

p1 = zeros(m,n,nPh); p2 = zeros(m,n,nPh); sgm2 = ones(nPh,1);
c = (1:nPh)/(nPh+1); 
load phi_cross_3t.mat;  U = compute_u0(phi_1,phi_2);
u = U;

h = fspecial('gaussian', 17, 4);
K1 = imfilter(ones(m,n), h, 'circular');

t0 = cputime;
for itr = 1:maxiter
    Uprev = U;
    Kb = imfilter(b,h, 'circular');
    Kb2 = imfilter(b.^2,h, 'circular');
    
    parfor k = 1:nPh
        Uk = squeeze(U(:,:,k));
        Ukx = [diff(Uk,1,2), zeros(m,1)]; Uky = [diff(Uk,1,1); zeros(1,n)];
        
        w1 = p1(:,:,k) + tau*Ukx;
        w2 = p2(:,:,k) + tau*Uky;
        wnorm= max(1, sqrt(abs(w1).^2+abs(w2).^2));
        w1 = w1./wnorm; w2 = w2./wnorm;
        
        p1(:,:,k) = w1; p2(:,:,k) = w2;
        DivW = [w1(:,1),diff(w1,1,2)] + [w2(1,:);diff(w2,1,1)];
        
        U(:,:,k) = Uk - theta*(0.5/wLength*(log(2*pi*sgm2(k)).*K1 + 1/sgm2(k).*(f.^2.*K1-2*c(k)*f.*Kb+c(k)^2*Kb2)) - DivW);
    end;
    
    U = projsplx2(U);
    
    parfor k = 1:nPh
        b_numer(:,:,k) = c(k)*U(:,:,k)/sgm2(k);
        b_denom(:,:,k) = c(k)*b_numer(:,:,k);
        
        c(k) = sum(sum(U(:,:,k).*f.*Kb))/sum(sum(U(:,:,k).*Kb2));
        sgm2(k) = 2*pi*sum(sum(U(:,:,k).*(f.^2.*K1-2*c(k)*f.*Kb+c(k)^2*Kb2)))/sum(sum(U(:,:,k).*K1));
    end
    
    b = imfilter(f.*sum(b_numer,3),h,'circular')./imfilter(sum(b_denom,3),h,'circular');
    
    if relerr(Uprev, U) < relchg_tol, break; end
    
end
  fprintf('itr = %.0f, RelerrU = %f, CPU = %f\n', itr, relerr(Uprev, U), cputime - t0);
[~, idx] = max(U,[],3);

hs0 = []; hs = [];
for k=1:nPh,
     hs0 = [hs0, u(:,:,k)];
     hs = [hs, U(:,:,k)];
end;

J = zeros(size(f));
for i = 1:nPh
    J = J + U(:,:,i).*c(i);
end

%% show results

figure;
subplot(221); imshow(f, []); title('Input image');
subplot(222); imshow(b.*M, []); title('Recovered b');
subplot(223); imshow(f.*M./b,[]); title('Bias Corrected image');
subplot(224); imshow(J.*M, []); title('Segmentation');

figure; imshow(cat(1,hs0,hs),[]);
title('K-mean initial (top) and segmentation result (bottom)');

