function x = projsplx2(y)
% project an n-dim vector y to the simplex Dn
% Dn = { x : x n-dim, 1 >= x >= 0, sum(x) = 1}

% (c) Xiaojing Ye
% xyex19@gmail.com
%
% Algorithm is explained as in the linked document
% http://arxiv.org/abs/1101.6081
% or
% http://ufdc.ufl.edu/IR00000353/
%
% Jan. 14, 2011.
%
% Modified by Xiaojing Ye on Jun. 15, 2011, convenient use in
% 2D image applications where y is a matrix of size m1 x m2 x n

[m1, m2, n]= size(y); 
iend = true;
N = m1*m2;

bget = false(m1,m2);

s = sort(y, 3, 'descend'); 
tmpsum = -ones(m1,m2);
t = zeros(size(bget));

for ii = 1:n-1
    
    tmpsum = tmpsum + s(:,:,ii);
    tmax = tmpsum/ii;
    ind = find(tmax >= s(:,:,ii+1));
    
    if ~isempty(ind)
        bget(ind) = true;
        t(ind) = tmax(ind);
        tmpsum(ind) = -1;
        all_ind = repmat(ind,[1,n]) + kron(N*ones(length(ind),1),0:n-1);
        s(all_ind) = 0;
    end;
    
    if sum(bget) == N
        iend = false;
        break;
    end;
    
end;
    
if iend
    ind = find(bget == 0);
    t(ind) = (tmpsum(ind)+s(ind+N*(n-1)))/n;
end
    
t = repmat(t, [1,1,n]);
x = max(y-t,0);

return;